/*
Copyright (C) 2023 Brian Evans (aka sloum). All rights reserved.

This source code is available under the terms of the ffsl, or,
Floodgap Free Software License. A copy of the license has been
provided as the file 'LICENSE' in the same folder as this source
code file. If for some reason it is not present, you can find the
terms of version 1 of the FFSL at the following URL:

https://www.floodgap.com/software/ffsl/license.html
*/

package main

import (
	"fmt"
	"io/ioutil"
	"os"
	"os/user"
	"path/filepath"
	"strings"
)

func readFile(path string) (string, error) {
	b, err := ioutil.ReadFile(ExpandedAbsFilepath(path))
	if err != nil {
		return "", fmt.Errorf("Unable to open file %s\n", path)
	}
	return string(b), nil
}

func ExpandedAbsFilepath(p string) string {
	if strings.HasPrefix(p, "~") {
		if p == "~" || strings.HasPrefix(p, "~/") {
			homedir, _ := os.UserHomeDir()
			if len(p) <= 2 {
				p = homedir
			} else if len(p) > 2 {
				if p[1] == '/' {
					p = filepath.Join(homedir, p[2:])
				} else {
					p = filepath.Join("/home", p[1:])
				}
			}
		} else {
			i := strings.IndexRune(p, '/')
			var u string
			var remainder string
			if i < 0 {
				u = p[1:]
				remainder = ""
			} else {
				u = p[1:i]
				remainder = p[i:]
			}
			usr, err := user.Lookup(u)
			if err != nil {
				p = filepath.Join("/home", u, remainder)
			} else {
				p = filepath.Join(usr.HomeDir, remainder)
			}
		}
	} else if !strings.HasPrefix(p, "/") {
		wd, _ := os.Getwd()
		p = filepath.Join(wd, p)
	}

	path, _ := filepath.Abs(p)
	return path
}
